#define FUNCTION_LENGTH 1000
#define OVERFLOW_RATE 2000000/256
//#define UPDATE_RATE OVERFLOW_RATE/1
#define UPDATE_RATE OVERFLOW_RATE/400
//#define UPDATE_RATE OVERFLOW_RATE/200

//routines for handling the incomming messages over the UART
void handle_message_length_1(unsigned char *msg_buffer);
void handle_message_length_2(unsigned char *msg_buffer);
void handle_message_length_3(unsigned char *msg_buffer);
void handle_message_length_5(unsigned char *msg_buffer);
void handle_message_length_52(unsigned char *msg_buffer);

//routines for updating display, computing new pattern indices
void Update_display(void);
void increment_index_x(void);
void increment_index_y(void);
void decrement_index_x(void);
void decrement_index_y(void);
void fetch_display_frame(unsigned short f_num);
void update_ANOUT(void);

//helper utilities
void toggle_trigger(void);
void delay(unsigned short us);
void long_delay(unsigned short ms); 
void send_number(unsigned char target_panel_addr, long long number);
void set_pattern(unsigned char pat_num);
void benchmark_pattern(void);


static unsigned char RESET[2] = {0x00, 0x01};
static unsigned char DISPLAY[2] = {0x00, 0x02};

static unsigned char TEST_BYTE[8][1] = {0x0F, 0xF0, 0xAA, 0x55, 0x03, 0x0C, 0x30, 0xC0};
static unsigned char TEST_3BYTE[9][3] = {{0x00, 0x00, 0x00}, {0x00, 0x00, 0xFF}, {0x00, 0xFF, 0x00}, {0x00, 0xFF, 0xFF},
											{0xFF, 0x00, 0x00}, {0xFF, 0x00, 0xFF}, {0xFF, 0xFF, 0x00}, {0xFF, 0xFF, 0xFF} , {0x0F, 0x33, 0x55} };

static unsigned char ALL_OFF[8] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
static unsigned char ALL_ON[8] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};

static unsigned char G_LEVELS[8][24] = {{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
 					{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF},
 					{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
 					{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF},
 					{0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
 					{0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF},
 					{0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
 					{0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF} };

static unsigned char ERROR_CODES[8][8] = { {0x80, 0x3F, 0x21, 0x21, 0x00, 0x3F, 0x25, 0x21},
 					{0x00, 0xBF, 0x21, 0x21, 0x00, 0x3F, 0x25, 0x21},
 					{0x00, 0x3F, 0xA1, 0x21, 0x00, 0x3F, 0x25, 0x21},
 					{0x00, 0x3F, 0x21, 0xA1, 0x00, 0x3F, 0x25, 0x21},
 					{0x00, 0x3F, 0x21, 0x21, 0x80, 0x3F, 0x25, 0x21},
					{0x00, 0x3F, 0x21, 0x21, 0x00, 0xBF, 0x25, 0x21},
					{0x00, 0x3F, 0x21, 0x21, 0x00, 0x3F, 0xA5, 0x21},
					{0x00, 0x3F, 0x21, 0x21, 0x00, 0x3F, 0x25, 0xA1} };
					
/*
static unsigned char PATTERNS[8][8] = { {0x3C, 0x42, 0x81, 0x81, 0x8F, 0x81, 0x42, 0x3C},
 					{0x3C, 0x42, 0x81, 0x81, 0x89, 0x85, 0x42, 0x3C},
 					{0x3C, 0x42, 0x81, 0x81, 0x89, 0x89, 0x4A, 0x3C},
 					{0x3C, 0x42, 0x81, 0x81, 0x91, 0xA1, 0x42, 0x3C},
					{0x3C, 0x42, 0x81, 0x81, 0xF1, 0x81, 0x42, 0x3C},
					{0x3C, 0x42, 0xA1, 0x91, 0x81, 0x81, 0x42, 0x3C},
					{0x3C, 0x52, 0x91, 0x91, 0x81, 0x81, 0x42, 0x3C},
					{0x3C, 0x42, 0x85, 0x89, 0x81, 0x81, 0x42, 0x3C} };
					
static unsigned char PAT_DATA[8][8] = { {240, 240, 240, 240, 15, 15, 15, 15},
 					{15, 240, 240, 240, 240, 15, 15, 15},
 					{15, 15, 240, 240, 240, 240, 15, 15},
 					{15, 15, 15, 240, 240, 240, 240, 15},
 					{15, 15, 15, 15, 240, 240, 240, 240},
					{240, 15, 15, 15, 15, 240, 240, 240},
					{240, 240, 15, 15, 15, 15, 240, 240},
					{240, 240, 240, 15, 15, 15, 15, 240} };
*/